import click

from nxstart import app
from nxstart.utils.files import get_full_path


@click.command()
@click.option('--name', prompt=True)
@click.option('--author', prompt=True)
@click.option('--clion', prompt=True)
def app_test_wrapper(name, author, clion):
    app.cpp(name, author, clion, get_full_path(''))


import os
import shutil

from click.testing import CliRunner

from nxstart.tests.base import app_test_wrapper
from nxstart.utils.files import get_full_path


def test_with_clion():
    runner = CliRunner()
    result = runner.invoke(app_test_wrapper, input='Test Project\nRuud Schroën\ny\n')
    cleanup()
    assert not result.exception
    assert result.output.endswith('Successfully created the libnx project!\n')


def cleanup():
    folder = get_full_path('test_project')
    if os.path.exists(folder):
        shutil.rmtree(get_full_path('test_project'))



import click
import os

from nxstart import app


class Context(object):

    def __init__(self):
        self.name = None
        self.author = None
        self.cwd = os.getcwd()


pass_context = click.make_pass_decorator(Context, ensure=True)


@click.group()
@click.option('--name', '-n', default=None, prompt='Project name', help='The name of your project')
@click.option('--author', '-a', default=None, prompt='Author name',
              help='The full name of the author')
@pass_context
def cli(ctx, name, author):
    ctx.name = name
    ctx.author = author


@cli.command('cpp', short_help='generate a libnx project')
@click.option('--clion/--no-clion', default=False, prompt='Are you using CLion?', help='include CMakeLists.txt')
@pass_context
def cpp(ctx, clion):
    app.cpp(ctx.name, ctx.author, clion, ctx.cwd)



# -*- coding: utf-8 -*-

"""Includes functions for copying the libnx template files."""

import datetime
import os
import shutil
from distutils.dir_util import copy_tree

from nxstart.utils.files import get_full_path, replace_in_file


def create_libnx_project(folder_path, name, author):
    """
    Copies the files from templates/base to folder_path and modifies Makefile and source/main.cpp
    to include the project name, author name and current date.

    :param folder_path: Path to copy the files to
    :param name: Name of the project
    :param author: Name of the author
    """
    template_folder = get_full_path(os.path.join('templates', 'cpp'))
    copy_tree(template_folder, folder_path)

    main_cpp_file = os.path.join(folder_path, 'source', 'main.cpp')
    main_cpp_replacements = {
        'APP_AUTHOR_PLACEHOLDER': author,
        'APP_NAME_PLACEHOLDER': name,
        'DATE_PLACEHOLDER': datetime.datetime.now().strftime("%Y-%m-%d")
    }
    replace_in_file(main_cpp_file, main_cpp_replacements)

    makefile = os.path.join(folder_path, 'Makefile')
    makefile_replacements = {
        'APP_NAME_PLACEHOLDER': name,
        'APP_AUTHOR_PLACEHOLDER': author
    }
    replace_in_file(makefile, makefile_replacements)


def create_cmake_lists_file(folder_path, folder_name):
    """
    Copies the CMakeLists.txt file from the templates folder to folder_path,
    and will use folder_name as the project name.

    This is useful for people who use CLion as their IDE.

    :param folder_path: Path to copy the file to
    :param folder_name: Project folder name
    """
    template_cmake_lists_file = get_full_path(os.path.join('templates', 'CMakeLists.txt'))
    shutil.copy2(template_cmake_lists_file, folder_path)

    cmake_lists_file = os.path.join(folder_path, 'CMakeLists.txt')
    cmake_lists_file_replacements = {
        'FOLDER_NAME_PLACEHOLDER': folder_name
    }
    replace_in_file(cmake_lists_file, cmake_lists_file_replacements)



# -*- coding: utf-8 -*-

"""Includes generic functions such as copying the README.md file."""

import os
import shutil

from nxstart.utils.files import get_full_path, replace_in_file


def create_readme_file(folder_path, name):
    """
    Copies the README.md file from the templates folder to folder_path,
    and will use name as the title of the document.

    :param folder_path: Path to copy the file to
    :param name: Project name to use as the title
    """
    template_readme_file = get_full_path(os.path.join('templates', 'README.md'))
    shutil.copy2(template_readme_file, folder_path)

    new_readme_file = os.path.join(folder_path, 'README.md')
    new_readme_file_replacements = {
        'APP_NAME_PLACEHOLDER': name
    }
    replace_in_file(new_readme_file, new_readme_file_replacements)






from os.path import join, dirname

PROJECT_ROOT = dirname(dirname(__file__))


def get_full_path(*path):
    """
    Get full path within project directory.

    :param path: Path to join with PROJECT_ROOT
    """
    return join(PROJECT_ROOT, *path)


def replace_in_file(file, replacements):
    """
    Replace strings with the replacements in the given file.

    :param file: File to replace strings in
    :param replacements: {string to replace: replacement, }
    """
    lines = []
    with open(file) as infile:
        for line in infile:
            for src, target in replacements.items():
                line = line.replace(src, target)
            lines.append(line)
    with open(file, 'w') as outfile:
        for line in lines:
            outfile.write(line)



# coding=utf-8

from setuptools import setup, find_packages

import os

dir_path = os.path.dirname(os.path.realpath(__file__))

# Get the version number
version = {}
with open(os.path.join(dir_path, 'nxstart', 'version.py')) as fp:
    exec(fp.read(), version)


def readme():
    with open('README.rst') as f:
        return f.read()


setup(
    name='nxstart',
    description='Nintendo Switch homebrew project generator',
    long_description=readme(),
    long_description_content_type='text/markdown',
    version=version['__version__'],
    url='https://github.com/roedesh/nxstart',
    author='Ruud Schroën',
    license='MIT',
    keywords='nintendo switch libnx nx homebrew project generator',
    packages=find_packages(),
    include_package_data=True,
    install_requires=[
        'Click'
    ],
    entry_points='''
        [console_scripts]
        nxstart=nxstart.cli:cli
    ''',
    project_urls={
        'Bug Reports': 'https://github.com/roedesh/nxstart/issues',
        'Source': 'https://github.com/roedesh/nxstart',
    },
    classifiers=[
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.6',
    ],
)



import os

import click

from nxstart import filebuilder


def cpp(name, author, clion, cwd):
    """
    Command that generates a libnx project.

    :param name: Name of the project
    :param author: Name of the author
    :param clion: Using CLion
    :param cwd: Current working directory
    """
    folder_name = name.lower().replace(" ", "_")
    folder_path = os.path.join(cwd, folder_name)

    if os.path.exists(folder_path):
        click.echo('Directory at %s is not empty.' % folder_path, err=True)
        raise click.Abort()

    os.makedirs(folder_path)
    filebuilder.cpp.create_libnx_project(folder_path, name, author)
    filebuilder.generic.create_readme_file(folder_path, name)

    if clion:
        filebuilder.cpp.create_cmake_lists_file(folder_path, folder_name)

    click.echo("Successfully created the libnx project!")
